        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         crcs.asm
; Function:     These are the routines for the various CRCs needed.
; Routines:     crcini: 16-bit CRC Initialize (for 1-wire)
;               crcchk: 16-bit CRC See if CRC passes (for 1-wire)
;               docrc:  16-bit CRC Cumulative calculation (for 1-wire)
;
;               docrc8: 8-bit CRC Cumulative calculation (no ini and check)
;
;               crc16b: 16-bit CRC calculation for Packet (also known as FCS)
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      crcini
; Called By:    iqwspd,iswich
; Calls:        None
; Gazintas:     None
; Gazoutas:     crc16h,crc16l==0
; Function:     Pretty cheasy routine to initialize the 16-bit CRC to all 0's.
; -----------------------------------------------------------------------------
crcini: clr     crc16l          ; clear LSB
        clr     crc16h          ; clear MSB
        rts                     ; done

; -----------------------------------------------------------------------------
; Routine:      crcchk
; Called By:    iqwspd,iswich
; Calls:        None
; Gazintas:     crc16h,crc16l (current CRC state
; Gazoutas:     Z set if OK, clear otherwise
; Function:     This checks to see if the resultant CRC is 0xB001.  This is
;               the correct result after a completed transaction.
;               After checking, Z is set if the CRC is correct
; -----------------------------------------------------------------------------
crcchk: lda     #$01            ; LSB comparison
        eor     crc16l          ; see if they match
        bne     crccdn          ; if not, in trouble
        lda     #$B0            ; MSB to compare to
        eor     crc16h          ; see if they match
crccdn: rts                     ; return with Z bit set as needed

        page
; -----------------------------------------------------------------------------
; Routine:      docrc
; Called By:    i_writ, i_read
; Calls:        None
; Gazintas:     crc16h, crc16l (CRC state before byte)
;               A: Byte to be munged into CRC
; Gazoutas:     crc16h, crc16l (CRC state after munging byte
; Function:     This is the CRC computation code.  There are lots of pretty
;               pictures about how the shift registers worked to compute the
;               CRC, but I just took the example C code (below) and converted
;               it to 6808 assembly.  It worked, so I didn't mess with it any
;               more.  The CRC processing is done on a byte-by-byte basis and
;               is called automatically by the byte write and byte read
;               commands.
;
;               static short oddparity[16] = {0,1,1,0,1,0,0,1,1,0,0,1,0,1,1,0};
;
;               unsigned short crc16(int data)
;               {
;                 data = (data ^ (CRC16 & 0xff)) & 0xff;
;                 CRC16 >>= 8;
;
;                 if (oddparity[data & 0xf] ^ oddparity[data >> 4])
;                 CRC16 ^= 0xc001;
;
;                 data <<= 6;
;                 CRC16   ^= data;
;                 data <<= 1;
;                 CRC16   ^= data;
;
;                 return CRC16;
;               }
; -----------------------------------------------------------------------------
docrc:  sta     crchld          ; remember value to I can restore it
        eor     crc16l          ; exor with CRC LSB
        sta     crcdat          ; save the data to crc for a bit

        lda     crc16h          ; Get CRC MSB
        sta     crc16l          ; save in CRC LSB
        clr     crc16h          ; clear out MSB

        lda     #8              ; want 8 bits
        sta     crccnt          ; save it
        lda     crcdat          ; get the data back
        clrx                    ; clear our bit counter
crclp:  rora                    ; shift a bit into C
        bcc     crcnb           ; if c=0, don't count
        incx                    ; if c=1, increment
crcnb:  dec     crccnt          ; decrement bit counter
        bne     crclp           ; do all eight bits
        stx     crccnt          ; save result in memory

        brclr   0,crccnt,crcsh  ; if even parity, go on to shifting

        lda     #$01            ; stuff to eor with CRC LSB
        eor     crc16l          ; do it
        sta     crc16l          ; save result
        lda     #$c0            ; stuff to eor with CRC MSB
        eor     crc16h          ; do it
        sta     crc16h          ; save result

crcsh:  lda     crcdat          ; get the data back
        clrx                    ; X is the CRC MSB
        asla                    ; shift LSB right (data << 1)
        rolx                    ; MSB too
        asla                    ; shift LSB right (data << 2)
        rolx                    ; MSB too
        asla                    ; shift LSB right (data << 3)
        rolx                    ; MSB too
        asla                    ; shift LSB right (data << 4)
        rolx                    ; MSB too
        asla                    ; shift LSB right (data << 5)
        rolx                    ; MSB too
        asla                    ; shift LSB right (data << 6)
        rolx                    ; MSB too

        sta     crccnt          ; save shifted data for a bit
        eor     crc16l          ; exor with CRC LSB
        sta     crc16l          ; save result
        txa                     ; get MSB of shifted data
        eor     crc16h          ; exor with CRC MSB
        sta     crc16h          ; save result
        lda     crccnt          ; get shifted data back

        asla                    ; shift LSB right (data << 7)
        rolx                    ; MSB too

        sta     crccnt          ; save shifted data for a bit
        eor     crc16l          ; exor with CRC LSB
        sta     crc16l          ; save result
        txa                     ; get MSB of shifted data
        eor     crc16h          ; exor with CRC MSB
        sta     crc16h          ; save result
        lda     crccnt          ; get shifted data back

        lda     crchld          ; restore original data byte
        rts                     ; all done (I think)

        page
; -----------------------------------------------------------------------------
; Routine:      docrc8
; Called By:    isrch,iqtmp2
; Calls:        None
; Gazintas:     crc8 (crc state before munging)
;               A data to munge into CRC
; Gazoutas:     crc8 (crc state after munging)
; Function:     This is the simpler 8-bit CRC that is used for one-wire
;               addresses and the temperature sensors.  The C code I copied
;               here simply chews up 256 bytes in a lookup table, so I just
;               copied it.  The table is called crc8tb in tables.asm.
;
;               C code (pretty trivial)
;               uchar dowcrc(uchar x)
;               {
;               DOWCRC = dscrc_table[DOWCRC ^ x];
;               return DOWCRC;
;               }
;
;               On initialization set the CRC8 value to 0, it should be 0 after
;               adding in the crc value from the source.
; -----------------------------------------------------------------------------
docrc8: eor     crc8            ; exor data with current crc8 value
        tax                     ; make an index
        clrh                    ; make sure H is 0
        lda     crc8tb,x        ; get new CRC value from table.
        sta     crc8            ; store result
        rts                     ; done.

        page
; -----------------------------------------------------------------------------
; Routine:      crc16b
; Called By:    TBD
; Calls:        None
; Gazintas:     crc16h, crc16l (CRC state before byte)
;               A: Byte to be munged into CRC
; Gazoutas:     crc16h, crc16l (CRC state after munging byte
; Function:     This is a faster CRC16 than the one used before.  For the TNC
;               support, I need a faster CRC16 algorithm and will burn the
;               memory for the table to do so (512 bytes).  This is a classic
;               lookup-based CRC calculation.
;
;               To minimize initial purtebations to the code I'm just going to
;               add this routine, later I'll pull out the previous one.
;
; -----------------------------------------------------------------------------
crc16b: eor     crc16l          ; xor incoming data with old CRC LSB
        clrh                    ; index MSB=0
        asla                    ; index times two
        tax                     ; make an index
        bcs     c16thi          ; if 2nd 256 bytes, cheat a bit
        lda     crc16t+1,x      ; Get table index LSB
        ldx     crc16t,x        ; get MSB (and trash index in process)
        bra     c16nxt          ; on to next

c16thi: lda     crc16t+257,x    ; get table index LSB (2nd 256 bytes)
        ldx     crc16t+256,x    ; get MSB (and trash index in process)

c16nxt: eor     crc16h          ; Xor table LSB with old CRC MSB
        sta     crc16l          ; save to new CRC LSB
        stx     crc16h          ; save Table LSB to new CRC MSB
        rts                     ; done

