        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         mx614.asm
; Function:     MX-614 Modem drivers
; Routines:     mxinit  Buffer initialize, puts in fixed data.
;               mxmsg   Broadcast a null-terminated message
;               mxqdat  Que up null-terminated data in (msg) to the Tx buffer
;               mxqstr  Que up null-terminated data in (hx)  to the Tx buffer
;               mxqndt  Que up a # of bytes of data in (msg) to the TX buffer
;               mxqnst  Que up a # of bytes of data in (hx)  to the TX buffer
;               mxterm  Terminate a packet message (CRC+Flag)
;               mxgo    Fire up state machine to transmit data
;               mxtirq  MX transmitter state machine timer code
;                       (see state machine states described in equates below)
;               mxsirq  MX transmitter bit-sending interrupt service routine
;               mxiqen  Call to conditionally re-enable interrupts in timer code
;               mxpten  If MX614 present, enable GPIOs for it.
; -----------------------------------------------------------------------------
; These are the routines to drive the MX-614 modem.  They are very much
; written as add-on routines to the existing UART drivers.  The biggest
; difference is that for the UART drivers I piecmeal the message together.  For
; the MX-614 I need the entire message ready before I start sending anything.
; This is because I have to feed the data to the MX-614 a bit/byte at a time
; and cannot ever be "late" delivering a bit.
;
; Calling mxinit clears out the buffer for the entire line of data and
; re-initializes it with the fixed data (all except WX data and CRC).  Every
; time data is ready for the UART driver, it calls mxqdat to que up the data.
; After the entire line of data has been queued up, call mxterm to finsish up
; the line's framing bytes and calling mxgo fires off the transmitter state
; machine.
;
; The transmitter state machine runs as a task under the timer interrupt.
; However, except for the semaphore to fire it off, it does run completely
; independently of any of the "regular" timer code.  They just happen to share
; the 100ms interrupt so they can do their sequencing every 100ms.
; When mxgo is called it waits for 500ms of no receive data from the radio
; before asserting the transmit line and starting to send continuous SYNC bytes.
; We stay in that mode for 300ms waiting for the transmitter to come up.  It
; Then the timer code sets the flag to allow the data state machinet to start
; sending the packet.
; The actual data transmission runs as a seperate interrupt service routine that
; handle each bit as an interrupt based on the falling edge of the MX614's
; /RDY line.  After the last bit is sent out, the bit state machine flags the
; timer interrupt routine it is done and it shuts down the transmitter and then
; also goes into an idle state.
;
; The data buffer consists of a two-byte pointer to the "next" byte to write
; and a 135-byte buffer.
; -----------------------------------------------------------------------------
; MX-614 connections as follows:
;
; PTC4 = DET (in) High=channel busy
; PTD0 = M0 (out) High=Transmit, Low= receive
; PTD1 = TXD (out) Transmit data
; PTD2 = RXD (in) Receive data
; PTD3 = CLK (out) Data clock
; PTD5 = RDY* (in) Low=ready for next bit (falling edge IRQ)
;
; I had designs on using the SPI controller, but the bit-by-bit requriements
; blew that theory.  I now just use the lines in GPIO mode.
; -----------------------------------------------------------------------------
;MXSTAT is the MX-614 state machine state variable.  States are as follows:
mxsidl: equ     0       ; Idle
mxswfs: equ     1       ; Waiting for silence
mxston: equ     2       ; Transmitter On
mxswft: equ     3       ; Wait for transmitter to start
mxstx:  equ     4       ; Transmitting (MX614 interrupts going on here)
mxswfo: equ     5       ; Transmitting done (and wait)
mxstof: equ     6       ; Transmitter off
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      mxinit
; Called By:    iqurt1
; Calls:        None.
; Gazintas:     None.
; Gazoutas:     mxptr updated
; Function:     This routine initializes the mxbuf buffer with the initial
;               fixed part of of a packet and sets mxptr so that the WX data
;               will get appended on to the end.  For now I am going to redo
;               this fixed data every packet.  I should trust the SRAM, but
;               for now I'm not.
; -----------------------------------------------------------------------------
mxinit: ldhx    #mxbuf          ; get the starting address
        stx     mxptr+1         ; save LSB
        pshh                    ; push H
        pulx                    ; get back in X
        stx     mxptr           ; save MSB

        ldhx    #dstsid         ; Framing byte and Destination callsign
        lda     #8              ; 8 bytes (header plus 7-byte destination ID
        jsr     mxqnst          ; queue it up
        lda     #$70            ; SSID of 0 with R bits set to 1 (pre-shift)
        sta     mxbuf+7         ; make Destination SSID 0

        ldhx    #fmycal         ; source callsign
        lda     #7              ; 7 bytes
        jsr     mxqnst          ; queue it up

        lda     fuityp          ; get the UI call type
        cmp     #2              ; see if WIDE 121212
        beq     mxwid2          ; if so go do wide processing
        cmp     #3              ; see if WIDE 121213
        beq     mxwid3          ; if so go do wide processing
        bra     mxusr           ; otherwise (including unprogrammed), user fixed

mxwid2: ldhx    #ww2tbl         ; point to WIDE 121212 table
        bra     mxwide          ; do wide processing
mxwid3: ldhx    #ww3tbl         ; point to WIDE 121213 table
mxwide: pshx                    ; save table LSB
        pula                    ; get it in A
        add     wwcnt           ; increment by index count
        adc     wwcnt           ; add twice since table is 16-bit
        psha                    ; save LSB
        pulx                    ; put it back in X
        pshh                    ; save MSB
        pula                    ; get it in A
        adc     #0              ; add the carry
        psha                    ; save MSB
        pulh                    ; put in H.  HX is now index to message!
        lda     ,x              ; get the message MSB
        ldx     1,x             ; get the LSB (destroys index, but OK)
        psha                    ; push MSB
        pulh                    ; HX now points to the message

        lda     #7              ; want to copy 7 bytes for WIDE message
        jsr     mxqnst          ; queue it up

        lda     wwcnt           ; get the count again
        inca                    ; increment it
        sta     wwcnt           ; save it (assuming it's OK)
        cmp     #5              ; compare to max value
        bls     mxinls          ; if still valid, done
        clra                    ; otherwise 0
        sta     wwcnt           ; start cycle over again
        bra     mxinls          ; on to other stuff in path

mxusr:  lda     fr1cal          ; Get repeater 1 callsign
        cmp     #$20            ; see if anything
        beq     mxinls          ; if not, all done
        cmp     #$ff            ; see if anything
        beq     mxinls          ; if not, all done

        ldhx    #fr1cal         ; repeater 1 callsign
        lda     #7              ; 7 bytes
        jsr     mxqnst          ; queue it up

        lda     fr2cal          ; Get repeater 2 callsign
        cmp     #$20            ; see if anything
        beq     mxinls          ; if not, all done
        cmp     #$ff            ; see if anything
        beq     mxinls          ; if not, all done

        ldhx    #fr2cal         ; repeater 2 callsign
        lda     #7              ; 7 bytes
        jsr     mxqnst          ; queue it up

; Now that we have copied the entire header, time to left shift it all and
; set the LSB of the last byte

mxinls: ldx     mxptr           ; end of string so far (MSB)
        pshx                    ; push it
        pulh                    ; get it in H
        ldx     mxptr+1         ; get LSB
mxinlp: aix     #-1             ; go back to last byte in header
        lsl     ,x              ; left shift the byte
        cphx    #mxbuf+1        ; see if we've made it to the beginning
        bne     mxinlp          ; loop till done

        ldx     mxptr           ; end of string so far (MSB)
        pshx                    ; push it
        pulh                    ; get it in H
        ldx     mxptr+1         ; get LSB
        aix     #-1             ; go back one (last byte of header)
        lda     ,x              ; get the byte
        ora     #$01            ; set the LSB
        sta     ,x              ; and save it back

; With the header done, add the Control and PID bytes

        ldhx    #tncd1          ; Command and PID bytes

        lda     #2              ; 2 bytes
        jsr     mxqnst          ; queue it up


;       **** DEBUG CODE ***
$IF TESTMODE
        ldhx    #tstmsg         ; Test Message
        jsr     mxqstr          ; queue it up
        jsr     mxterm          ; terminate the test string for transmission
        jsr     mxgo            ; fire things off

        lda     #30             ; 3 seconds
        jsr     waits           ; wait it for IRQs to do their thing.

xxxs:   bra     mxinit          ; go do it again
$ENDIF
        rts                     ; all done!

        page
; -----------------------------------------------------------------------------
; Routine:      mxmsg
; Called By:    main routine
; Calls:
; Gazintas:     HX points to message to send
; Gazoutas:     The message to the airwaves
; Function:     This routine sends a null-terminated message over the airwaves.
;               The message is only sent if we are in modem mode, so it is
;               save to call it without the caller needing to check.
; -----------------------------------------------------------------------------
mxmsg:  lda     fumode          ; get the UART mode
        cmp     #4              ; see of MX-614 mode
        bne     mxmex           ; if not, don't try to send

        pshh                    ; save message pointer
        pshx                    ; LSB too
        jsr     mxinit          ; initialize the MX614 buffer
        pulx                    ; restore message pointer
        pulh                    ; MSB too

        jsr     mxqstr          ; queue it up
        jsr     mxterm          ; terminate the string for transmission
        jsr     mxgo            ; fire things off

mxmwt:  brclr   3,portb,mxmex   ; DEBUG!! break out if up button pressed

        lda     mxstat          ; get the transmit state machine status
        cmp     #mxsidl         ; see if idle
        bne     mxmwt           ; wait until idle (message sent)

mxmex:  rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      mxqdat  mxqstr
; Called By:    irqmsg  mxinit
; Calls:        None.   None.
; Gazintas:     mxqdat: (msg)   Null-terminated string to add to mxbuf.
; Gazintas:     mxqstr: hx      Null-terminated string to add to mxbuf.
; Gazoutas:     (mxbuf) Updated to include what was in (msg)
; Function:     This routine copies bytes to the mxbuf buffer.  The source
;               buffer is null-terminated or $ff terminated
; -----------------------------------------------------------------------------
mxqdat: clrh                    ; 0...
        ldx     #msg            ; HX points to the start of msg
mxqstr: stx     mxiptr+1        ; save LSB
        pshh                    ; Save H
        pulx                    ; get into X
        stx     mxiptr          ; save MSB

mxqlp:  ldx     mxiptr          ; get MSB
        pshx                    ; push it
        pulh                    ; pop it
        ldx     mxiptr+1        ; get LSB, HX points to byte to copy
        lda     ,x              ; get it
        beq     mxqex           ; if 0, all done
        cmp     #$ff            ; make sure something there
        beq     mxqex           ; exit if nuthin
        aix     #1              ; next byte
        stx     mxiptr+1        ; save address LSB
        pshh                    ; Save H
        pulx                    ; get into X
        stx     mxiptr          ; save MSB

        ldx     mxptr           ; get MSB
        pshx                    ; push...
        pulh                    ; pop into H
        ldx     mxptr+1         ; HX now has (mxptr)
        cphx    #mxbnd          ; check for overflow
        beq     mxqex           ; if so, exit
        sta     ,x              ; save byte
        aix     #1              ; increment
        stx     mxptr+1         ; save LSB
        pshh                    ; push H
        pulx                    ; get back in X
        stx     mxptr           ; save MSB
        bra     mxqlp           ; on to next byte

mxqex:  rts                     ; done!

        page

; -----------------------------------------------------------------------------
; Routine:      mxqndt  mxqnst
; Called By:    TBD
; Calls:        None.   None.
; Gazintas:     mxqndt: (msg)   string to add to mxbuf.
; Gazintas:     mxqnst: hx      string to add to mxbuf.
;               Both:   A       number of bytes to copy.
; Gazoutas:     (mxbuf) Updated to include the string
; Function:     This routine copies bytes to the mxbuf buffer.  The number of
;               bytes to copy is specified in the A register.
;               This is also my first attempt at using a stack-based variable,
;               the byte count...
; -----------------------------------------------------------------------------
mxqndt: clrh                    ; 0...
        ldx     #msg            ; HX points to the start of msg
mxqnst: stx     mxiptr+1        ; save LSB
        pshh                    ; Save H
        pulx                    ; get into X
        stx     mxiptr          ; save MSB

        psha                    ; save byte count

mxqnlp: ldx     mxiptr          ; get MSB
        pshx                    ; push it
        pulh                    ; pop it
        ldx     mxiptr+1        ; get LSB, HX points to byte to copy
        lda     ,x              ; get it
        aix     #1              ; next byte
        stx     mxiptr+1        ; save address LSB
        pshh                    ; Save H
        pulx                    ; get into X
        stx     mxiptr          ; save MSB

        ldx     mxptr           ; get MSB
        pshx                    ; push...
        pulh                    ; pop into H
        ldx     mxptr+1         ; HX now has (mxptr)
        cphx    #mxbnd          ; check for overflow
        beq     mxqnx           ; if so, exit
        sta     ,x              ; save byte
        aix     #1              ; increment
        stx     mxptr+1         ; save LSB
        pshh                    ; push H
        pulx                    ; get back in X
        stx     mxptr           ; save MSB

        pula                    ; get byte count
        deca                    ; decrement it
        psha                    ; put it back
        beq     mxqnx           ; if 0, all done
        bra     mxqnlp           ; on to next byte

mxqnx:  pula                    ; get count off the stack
        rts                     ; done!

        page
; -----------------------------------------------------------------------------
; Routine:      mxterm
; Called By:    iqurt5
; Calls:        TBD
; Gazintas:     None.
; Gazoutas:     (mxbuf) trailer bytes added.
; Function:     This routine adds the CRC and trailing flag to the MX-614
;               message to be transmitted.  It also sets the last character
;               flag.  Note that by only checking the LSB of the last characters
;               address I have an inherent limit of a 256-byte message.  This
;               seems reasonable (famous last words??!!)
;
;               The CRC is calculated for all bytes after the flag byte and up
;               to the byte just before the CRC.
; -----------------------------------------------------------------------------
mxterm: lda     mxptr+1         ; get buffer pointer LSB
        sta     mxlst           ; save it for end-of-string flag

        ldhx    #mxbuf+1        ; the place to start
        sthx    mxtxad          ; save it
        lda     #$ff            ; CRC initialization value
        sta     crc16l          ; initialize LSB
        sta     crc16h          ; initialize MSB

mxtlp:  ldhx    mxtxad          ; get the next byte address
        cpx     mxlst           ; see if done yet
        beq     mxtcrc          ; if so, go insert CRC
        lda     ,x              ; get a byte
        aix     #1              ; increment pointer
        sthx    mxtxad          ; save the address
        jsr     crc16b          ; add it to the CRC
        bra     mxtlp           ; go do next byte

;       We have the CRC, but now we need to bit-reverse it.  In my quest for
;       efficiency I have a tricky way of loading all 16 bits into the A and
;       X regiters and by shifting between them, get all 16 bits reversed in
;       16 core instructions.  It basically involves essentially shifting
;       data simultaneously between A and X through the carry.  As each bit is
;       shifted into one register, the bit shifted out goes back into the
;       other register.  Cool, but almost impossible to explain in text

mxtcrc: lda     crc16l          ; get LSByte
        ldx     crc16h          ; get MSByte
        bra     mxnbi           ; bypass bit reversal
        clc                     ; start with a clear carry bit
        bit     #$01            ; check LSByte LSBit
        beq     mxtc0           ; skip setting Carry if LSBit is 0
        sec                     ; set carry if LSBit is 0

mxtc0:  rolx                    ; LSBit from LSByte into LSBit of MSByte (1)
        rora                    ; MSBit from MSByte into MSbit of LSByte (2)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (3)
        rora                    ; MSBit from MSByte into MSbit of LSByte (4)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (5)
        rora                    ; MSBit from MSByte into MSbit of LSByte (6)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (7)
        rora                    ; MSBit from MSByte into MSbit of LSByte (8)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (9)
        rora                    ; MSBit from MSByte into MSbit of LSByte (10)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (11)
        rora                    ; MSBit from MSByte into MSbit of LSByte (12)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (13)
        rora                    ; MSBit from MSByte into MSbit of LSByte (14)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (15)
        rora                    ; MSBit from MSByte into MSbit of LSByte (16)
        rolx                    ; LSBit from LSByte into LSBit of MSByte (17) (Last bit)

;       After all that jumbling A has the bit-reversed LSB and X the bit-reversed MSB
mxnbi:  coma                    ; invert bits ??? (copying TNC-X code)
        comx                    ; invert bits ??? (copying TNC-X code)
        sta     crc16l          ; save MSByte
        stx     crc16h          ; save LSByte

mxbyps: ldx     mxptr           ; get pointer MSB
        pshx                    ; save it
        pulh                    ; get into H
        ldx     mxptr+1         ; get LSB in X
        lda     crc16l          ; get MSB
        sta     ,x              ; save MSByte first (bit order reversed)
        lda     crc16h          ; get the LSByte
        aix     #1              ; increment pointer
        sta     ,x              ; save it
        lda     #$7e            ; end of frame flag
        aix     #1              ; increment pointer
        sta     ,x              ; save it in the string
        stx     mxptr+1         ; save LSB
        pshh                    ; push MSB
        pula                    ; get it back
        sta     mxptr           ; save it

        incx                    ; increment X only
        stx     mxlst           ; save last-byte flag
        rts                     ; done!

; -----------------------------------------------------------------------------
; Routine:      mxgo
; Called By:    iqurt5
; Calls:        None.
; Gazintas:     None.
; Gazoutas:     mxstat  updated to fire off the transmitter state machine
; Function:     This routine fires off the transmitter state machine.  This is
;               all interrpt driven, so all I do here is take the mxstat
;               semaphore from idle and set it to the first wait for silence
;               state.
; -----------------------------------------------------------------------------
mxgo:   lda     fumode          ; get the currentmode
        cmp     #4              ; make sure we are in Modem mode
        bne     mxgex           ; if not, don't fire up the machine!
        lda     ftxclr          ; get clear time count
        sta     mxscnt          ; save it.
        lda     #mxswfs         ; go to wait for silence state
        sta     mxstat          ; set it
mxgex:  rts                     ; done!

; -----------------------------------------------------------------------------
; Routine:      mxtirq
; Called By:    timrtn
; Calls:        None.
; Gazintas:     None.
; Gazoutas:     None.
; Function:     This routine manages the timer part of the mx-614 transmitter
;               state machine.  See the header at the top of the file for
;               a listing of the states and what they mean.
; -----------------------------------------------------------------------------
mxtirq: lda     mxstat          ; get the current state
        bne     mxtiac          ; if not idle, go do stuff
        jmp     mxtiex          ; if idle, take long jump to short cut out

mxtiac: cmp     #mxstof         ; see if greater than last state
        bhi     mxrtof          ; if so, something bad happened, reset xmitter

        deca                    ; 0 state (idle) already taken care of
        lsla                    ; multiply index by two
        tax                     ; make an index
        clrh                    ; MSB=0
        lda     mxtab,x         ; get target address MSB
        psha                    ; push it
        lda     mxtab+1,x       ; get target address LSB
        psha                    ; push it
        pulx                    ; pop LSB into X
        pulh                    ; pop MSB into H
        jmp     ,x              ; go to routine

mxtab:  fdb     mxrwfs          ; 01: Wait for silence
        fdb     mxrton          ; 02: Transmitter on (sending SYNCs)
        fdb     mxrwft          ; 03: Send SYNCs
        fdb     mxrtx           ; 04: Transmit data
        fdb     mxrwfo          ; 05: Transmit done (send SYNCs)
        fdb     mxrtof          ; 06: Transmitter off (send SYNCs)

; -----------------------------------------------------------------------------
; mxrwfs: Wait for no RX data (clear channel).
;         We sit here and poll the Rx busy bit every 100ms.  It must be clear
;         FTXCLR times (user programmable) in a row before we go enable the
;         transmitter in the next state.
;         If the channel is busy (or MX-614 not plugged in) for 25 seconds,
;         I assume something bad has happened and abort the transmission.
;         mxscnt is preloaded with the timeout time in 100ms increments.
;         The counter is decremented for each 100ms of quiet time until it
;         gets to 0.  It is incremented 100ms for each busy time too.  If
;         the busy count gets to 255 (25.5 seconds) we have a timeout.  To
;         keep things OK, if the counter is >5 for the decrement rouint, it
;         just gets preloaded with ftxclr-1 so even if busy, we still wait
;         only ftxclr counts.  I don't bother resetting on timeouts.
; -----------------------------------------------------------------------------
mxrwfs: brclr   4,portc,mxiwti  ; if MX-614 DETECT pin low, then line not busy
        lda     mxscnt          ; Line busy, get current count.
        inca                    ; plus one
        sta     mxscnt          ; save it
        inca                    ; increment once more so FF goes to 00
        bne     mxtiex          ; if no carry, didn't roll over, all done
        bra     mxrtof          ; otherwise timeout, shut down.

mxiwti: lda     mxscnt          ; otherwise not busy, get the state count
        cmp     ftxclr          ; compare to delay value
        bcs     mxiwtb          ; if less than ftxclr, just decrement
        lda     ftxclr          ; otherwise preload again with ftxclr (then dec)
mxiwtb: deca                    ; decrement it
mxiwts: sta     mxscnt          ; and save it
        bne     mxtiex          ; if not 0, all done

        lda     #mxston         ; if 0, waited TXDELAY, on to transmit on state
        sta     mxstat          ; set it
        bra     mxtiex          ; and exit

; -----------------------------------------------------------------------------
; mxrton: Turn on the transmitter and start sending SYNCs.
;         The bit state machine starts sending data as soon as it sees the
;         transmitter on (it looks at PTD0).  It then checks mxstat.  If it
;         isn't in mxrtx, then it just sends SYNCs.  Hence all we have to do
;         here is turn on the transmitter, the SYNCs are taken care of by
;         the bit state machine.
;         This state also fires off the bit state machine.  It is interrupt-
;         driven, so firing off the first bit here will generate an interrupt
;         as soon as the first bit is finished.  Subsequent bits are handled
;         by the interrupt service routine mxsirq.
; -----------------------------------------------------------------------------
mxrton: bset    0,portd         ; 1=turn on the transmitter
        lda     #mxswft         ; next state (wait for transmitter to warm up)
        sta     mxstat          ; save it

        ldhx    #mxbuf+1        ; buffer start (+1 as we pre-stuff 1st byte)
        sthx    mxtxad          ; save it

        lda     mxbuf           ; stuff first byte
        sta     mxobyt          ; save it
        lda     #$08            ; bit counter
        sta     mxobit          ; save it
        clr     mxsync          ; flag 1st byte as a sync

        bclr    1,portd         ; start with "0" data
        bset    3,portd         ; assert the clock (fires off things)
        bclr    3,portd         ; unassert the clock

        lda     ftxdly          ; Transmit on to send data delay (in 100ms)
        sta     mxscnt          ; save it

        bra     mxtiex          ; and exit

; -----------------------------------------------------------------------------
; mxrwft: Wait for the transmitter to warm up.
;         We sit here for FTXDLY-1 (user-programmable) states, each 100ms.
;         It is one less state because the first 100ms were used up in the
;         mxrton state.
;         mxscnt has already been pre-loaded with the number of states to wait.
; -----------------------------------------------------------------------------
mxrwft: lda     mxscnt          ; otherwise not busy, get the state count
        deca                    ; decrement it
        sta     mxscnt          ; and save it
        bne     mxtiex          ; if not 0, all done

        lda     #mxstx          ; if 0, on to next state
        sta     mxstat          ; set it
        bra     mxtiex          ; and exit

; -----------------------------------------------------------------------------
;mxrtx: Transmit the data.  Don't do anything here, MX614 interrupts are doing
;       the work.  When they are done, they will set the state to mxrwfo,
;       indicating the data part of the transmission is complete.
; -----------------------------------------------------------------------------
mxrtx:  bra     mxtiex          ; and exit

; -----------------------------------------------------------------------------
; mxrwfo: Stay here for one timer tick, do nothing else.  This is to guarantee
; at least a little bit of hold time on the transmitter being on after the
; last bit of data has been transmitted.
; -----------------------------------------------------------------------------
mxrwfo: lda     #mxstof         ; next state
        sta     mxstat          ; set it
        bra     mxtiex          ; and exit

; -----------------------------------------------------------------------------
; mxrtof: Turn the transmitter off and go back to the idle state.
;         Also go here if there is any kind of state machine state error
; -----------------------------------------------------------------------------
mxrtof: bclr    0,portd         ; 0=turn off the transmitter
        lda     #mxsidl         ; the idle state
        sta     mxstat          ; save it
        bra     mxtiex          ; and exit

mxtiex: rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      mxsirq
; Called By:    Falling edege of /RDY from MX-614
; Calls:        None
; Gazintas:     None directly
; Globals       mxoptr, mxobit
; Gazoutas:     None directly
; Function:     This routine is in charge of the bit-by-bit transmission of
;               the data to the MX614.  As part of the bit-oriented routines,
;               this routine does the bit stuffing and NRZI encoding.
;               The first part of the code dismisses the pending HW interrupt
;               state.  I check to see if we are transmitting (an error check),
;               and if so, abort out.  Next I check if we are in the middle of
;               sending a byte.  If so I jump to the byte sending routine,
;               otherwise I fall into the code that gets and queues up the
;               next byte.
;
;               The Byte figgerin' and Byte sendin' code have their own
;               comment blocks.
;
;               This routine is time critical.  It gets executed for every bit
;               in every byte, effciency matters here!
; -----------------------------------------------------------------------------
mxsirq: pshh                    ; save H (part of IRQ handling)
        lda     t1sc1           ; read T1SC1 with CH1F set
        mov     #$48,t1sc1      ; so this write clears the IRQ

        brclr   0,portd,mxsiqx  ; if transmitter off, abort out

;       Now that we are doing something, see if data queued up.
;       Note that mxrton (the routine that fires this code off), preloads
;       mxobit and mxobyt with a SYNC before enabling this interrupt.
mxsiqd: lda     mxobit          ; check bit counter
        beq     mxnbit          ; if 0, figger out next bit
        clra                    ; 0... (to compare mxsync to)
        cbeq    mxsync,mxybit   ; if it's a sync byte, don't bit stuff
        bra     mxsbit          ; if not 0, just go send the bits

; NEXT BYTE FIGGERIN HERE
; While transmitting, there are three states.  The "center" state is where the
; packet proper is being sent.  Before it and after it we are waiting for the
; transmitter to either warm up or cool down and during those states the next
; byte is by definition a SYNC byte.
; If we are in the packet proper, we first check to see if we are at the last
; byte and exit if so.  If not, then we got get the next byte and reset the bit
; counter.  If it is the last byte (a SYNC), I also set the mxsync flag to make
; sure it doesn't get bit stuffed.  With that all done, I just fall into the
; bit sending code.

mxnbit: lda     mxstat          ; get status
        cmp     #mxstx          ; see if regular transmission
        beq     mxndat          ; if so, go get data
        clr     mxsync          ; otherwise it's a sync byte
        lda     #$7e            ; a sync byte
        sta     mxobyt          ; save it too
        bra     mxinsy          ; go do regular processing

mxndat: ldhx    mxtxad          ; get the Address of byte to send
        cpx     mxlst           ; see if we're done
        beq     mxsdun          ; if so, exit gracefully

        lda     ,x              ; otherwise get the byte to transmit
        sta     mxobyt          ; save new byte to send
        mov     #1,mxsync       ; assume all bytes will not be sync's
        incx                    ; temporarily increment X to see if last byte
        cpx     mxlst           ; see if we're almost done (last sync byte)
        bne     mxnlst          ; if not, leave sync flag alone
        clr     mxsync          ; set sync flag so last sync byte stays OK
mxnlst: decx                    ; get X back to original value
        aix     #1              ; increment address
        sthx    mxtxad          ; save updated address
mxinsy: mov     #8,mxobit       ; update bit counter
        clra                    ; 0... (to compare mxsync to)
        cbeq    mxsync,mxybit   ; if it's a sync bit, don't bit stuff
;                               ; otherwise do regular processing

;       For the bitwise sending of data, I have this generic routine
;       that shifts out one bit at a time from mxobyt using mxobit as
;       a bit counter.  Wrinkling things a little is the 1-bit counter
;       to ensure we never get 6 1-bits in a row except for the sync
;       bytes.  If we do, a 0-bit is inserted.  If the caller knows
;       we are sending a sync byte, it jumps to mxybit instead of
;       mxsbit to bypass the 1-bit checking.

mxsbit: tst     mx1bit          ; see if we've sent 5 1 bits
        beq     mxsd0           ; if so, go send a 0

mxybit: dec     mxobit          ; decrement bit counter
        asr     mxobyt          ; get data LSB into carry
        bcc     mxsd0           ; if C 0, send 0

mxsd1:  ;                       ; For a 1-bit, don't change the tone
        dec     mx1bit          ; decrement 1-bit counter
        bra     mxsclk          ; on to clock if less than 5 bits

mxsd0:  ;                       ; For a 0-bit, change the tone.
        brclr   1,portd,mxsd00  ; check the tone state, if 0, set to 1
        bclr    1,portd         ; otherwise clear it
        bra     mxsdc5          ; and reset the 1's counter

mxsd00: bset    1,portd         ; if tone bit was 0, set to 1.
mxsdc5: mov     #5,mx1bit       ; reset the 1-bit counter

mxsclk: bset    3,portd         ; assert the MX614 clock (clears external IRQ)
        bclr    3,portd         ; unassert the clock

mxsiqx: pulh                    ; restore H (part of IRQ handling)
        rti                     ; all done

mxsdun: lda     #mxswfo         ; all done flag
        sta     mxstat          ; save it
        bra     mxsclk          ; exit (keeping data moving)

        page
; -----------------------------------------------------------------------------
; Routine:      mxiqen
; Called By:    timirq, i_rst,i_bit
; Calls:        None.
; Gazintas:     None.
; Gazoutas:     None.
; WARNING:      This routine must preserve A!
; Function:     This routine is in charge of conditionally re-enabling
;               interrupts.  This is called after some initial houscleaning at
;               the start of the timer interrupt to allow MX-614 interrupts.
;               Just before wiggling the i-button IO line interrupts are
;               unconditionally disabled (SEI) and the conditionally re-enabled
;               by calling this routine again.  This is done on a bit-by-bit
;               basis.  This routine needs to be as efficient as possible as it
;               gets called a lot in time-critical code.  Interrupts only get
;               re-enabled if there is data pending in the MX-614 buffer.  This
;               is flagged by the mxstate variable being "mxstxe".
; -----------------------------------------------------------------------------
mxiqen: tst     mxstat          ; check transmitter state machine status
        beq     mxiqdi          ; if idle, don't re-enable interrupts.
        cli                     ; otherwise re-enable interrupts

mxiqdi: rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      mxpten
; Called By:    init code, after MX-614 setup
; Calls:        None
; Gazintas:
; Gazoutas:
; Function:     This routine reconfigures the IO lines for the MX-614 if it
;               is present.  As some of these lines are inputs, I need to leave
;               them as outputs unless the MX-614 is installed so I don't have
;               floating inputs.
; -----------------------------------------------------------------------------
mxpten: lda     fumode          ; get the UART mode
        cmp     #4              ; see of MX-614 mode
        beq     mxptmx          ; if so, go set up IO for it.

        mov     #$ff,ddrc       ; all outputs
        mov     #$ff,ddrd       ; all outputs
        bra     mxptex          ; all done

mxptmx: mov     #$ef,ddrc       ; C4 input, rest outputs
        mov     #$1b,ddrd       ; D5,D2 inputs, rest outputs
        mov     #$48,t1sc1      ; Set up T1ch1 input irq, falling edge.

mxptex: rts
