        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         onewire.asm
; Function:     These are the routines for doing Dallas 1-wire communications
; Routines:     isrch:  Search for serial numbers given a family type
;               sndadr: Initialize bus and send family code
;               i_rst:  Send a reset pulse, see if anyone out there
;               i_bit:  Send/receive a bit
;               i_writ: Send a byte (doing 16-bit CRC)
;               i_read: Read a byte (doing 16-bit CRC)
;               i_rdb:  read a bit
;               i2srch: See if multiple IDs for same family code.
;               i2pick: Let user pick between two IDs.
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      isrch
; Called By:    ssens, iqdir2, iqdir3
; Calls:        i_rst (resets 1-wire bus)
;               i_writ (send a 1-wire byte)
;               i_rdb (read a 1-wire bit)
;               i_bit (write a 1-wire bit)
;               docrc8 (do 8-bit CRC)
; Gazintas:     A: The family code to search
;               flg12 (0=pick 0's if both, 1=pick 1's if both)
; Gazoutas:     mndray (found address, or 0's if none found)
; Function:     This is the family search routine.  Provide the family code you
;               want to search for in the A register, and this routine will go
;               out and see what it can find.  The result is placed in the eight
;               mndray bytes.  This routine is not a general purpose ID finding
;               routine, it has been significantly simplifed.  This routine is
;               only needed for searching out the wind direction chips, so I
;               know there are only two possible devices out there.  When we do
;               the searching, the flag flg12 decides how this routine resolves
;               multiple devices.  If flg12 is 0, this routine will choose a 0
;               when there is a conflict.  If flg12 is 1, this routine will
;               choose a 1 when there is a conflict.  Since there can be at most
;               two devices, calling this twice with the flag complemented will
;               get us either two readings the same, or the two unique readings
;               if two are activated.
;               This routine not only gets an address, but also checks to make
;               sure it is a valid address by doing an 8-bit CRC in the
;               received data.  If the address is good, we return, otherwise
;               we trie up to #tries times before giving up.  If there is an
;               error, we return all zeros for an address.
; -----------------------------------------------------------------------------
isrch:  sta     i_fam           ; store family code for use later.
        lda     #tries          ; # of tries
        sta     chrcnt          ; remember that
isretr: jsr     i_rst           ; go do a reset and see if anyone is there.
        beq     isrcgr          ; if answer, go read ROM address
        jmp     idrdnr          ; if no answer, long jump to error

isrcgr: lda     #$f0            ; Search ROM command
        jsr     i_writ          ; send it

        lda     i_fam           ; get family ID back
        sta     mndray          ; place we spool it from
        lda     #8              ; want to scan 8 bits
        sta     temp            ; save it

famrd1:
        jsr     i_rdb           ; go get first bit
        ror     i_fam1          ; store it

        jsr     i_rdb           ; go get 2nd bit
        ror     i_fam2          ; store it

        lda     mndray          ; get family code
        jsr     i_bit           ; ack only the family code I want
        sta     mndray          ; save result

        dec     temp            ; decrement our counter
        bne     famrd1          ; get 8 bits


;       Read in the rest of the family code:

        lda     #56             ; want to scan 56 bits
        sta     temp            ; save it

idrd1:
        jsr     i_rdb           ; go get first bit (true data)
        rol     temp2           ; save in temp2(0)
        jsr     i_rdb           ; go get 2nd (non-inverted) bit
        rol     temp2           ; put bit in temp2(0) (true in temp2(1)
        lda     temp2           ; get result back in A
        eor     #$01            ; invert inverted bit
        brset   0,flg12,idrdt   ; if flg12 is set, ack inverted (in a(0) now)
        rora                    ; otherwise toss inverted (true in A(0))
idrdt:  rora                    ; bit to ack in C now
        ror     mndray+7        ; shift the bit into storage
        ror     mndray+6        ; store it
        ror     mndray+5        ; store it
        ror     mndray+4        ; store it
        ror     mndray+3        ; store it
        ror     mndray+2        ; store it
        ror     mndray+1        ; store it

        lda     mndray+7        ; get bit to Ack
        rola                    ; shove into carry
        rola                    ; now in bit 0
        jsr     i_bit           ; ack only the family code I want

        dec     temp            ; decrement our counter
        bne     idrd1           ; get 56 bits

        clr     crc8            ; clear the crc8 regsiter
        lda     mndray+0        ; get byte 1
        jsr     docrc8          ; crc it
        lda     mndray+1        ; get byte 2
        jsr     docrc8          ; crc it
        lda     mndray+2        ; get byte 3
        jsr     docrc8          ; crc it
        lda     mndray+3        ; get byte 4
        jsr     docrc8          ; crc it
        lda     mndray+4        ; get byte 5
        jsr     docrc8          ; crc it
        lda     mndray+5        ; get byte 6
        jsr     docrc8          ; crc it
        lda     mndray+6        ; get byte 7
        jsr     docrc8          ; crc it
        lda     mndray+7        ; get byte 8
        jsr     docrc8          ; crc it

        lda     crc8            ; get crc result
        beq     idrex           ; if $00, a valid ID
idrdnr: clr     mndray+0        ; clear out ID
        clr     mndray+1        ; clear out ID
        clr     mndray+2        ; clear out ID
        clr     mndray+3        ; clear out ID
        clr     mndray+4        ; clear out ID
        clr     mndray+5        ; clear out ID
        clr     mndray+6        ; clear out ID
        clr     mndray+7        ; clear out ID

        dec     chrcnt          ; decrement the tries counter
        beq     idrex           ; if down to 0, we got big problems.
        jmp     isretr          ; if not, go try again

idrex:  rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      sndadr
; Called By:    iqtmp1, iqtmp2, iqwspd, iswich
; Calls:        i_rst
;               i_writ (send a byte)
; Gazintas:     A: family code to send
; Gazoutas:     None
; Function:     This routine simply sends the ibutton address pointed to by X.
;               I send the reset command, match ROM command, and then the
;               8 bytes of address.  This initiates a transaction with some
;               specific device.
; -----------------------------------------------------------------------------
sndadr: pshx                    ; save x
        pshh                    ; save H too
        bsr     i_rst           ; send the reset
        lda     #$55            ; match ROM command
        bsr     i_writ          ; send it off
        lda     #8              ; 8-byte counter
        sta     cntr            ; store it

sadlp:  pulh                    ; get H back
        pulx                    ; get X back
        lda     0,x             ; get the address byte
        aix     #1              ; increment pointer
        pshx                    ; save x
        pshh                    ; save H too
;       stx     temp            ; save it
        bsr     i_writ          ; send it
        dec     cntr            ; decrement the byte counter
        bne     sadlp           ; send all eight bytes
        pulh                    ; get H back
        pulx                    ; get X back
        clrh                    ; make sure H is clear
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      i_rst
; Called By:    rstrtn, isrch, sndadr
; Calls:        None
; Gazintas:     None
; Gazoutas:     Z: set if answer-to-reset pulse, cleared otherwise
; Function:     This routine issues an ibutton reset and sees if anyone
;               responds.  I assert the wire low for 500us, then sample it for
;               500us after that.  If, at any time during that 500us high time,
;               someone else drives the bus low, we return with Z bit set,
;               meaning someone answered.  500us is 1000 clocks.  Numbers in
;               parens is the instruction length.
; -----------------------------------------------------------------------------
i_rst:  tpa                     ; get CCR
        psha                    ; save it
        bset    0,ddrb          ; (5) drive bus low
$IF DEBUG
        ldx     #77             ; (2) 500us=77 loops x 16 clocks * 2.45MHz
$ELSEIF
        ldx     #250            ; (2) 500us=250 loops x 16 clocks * 8.00MHz
$ENDIF
irlp1:  decx                    ; (1) decrement our counter
        nsa                     ; (3) waste some time
        nsa                     ; (3) waste some more time
        nsa                     ; (3) waste some time
        nsa                     ; (3) waste some more time
        bne     irlp1           ; (3) wait

        bclr    0,ddrb          ; (5) unassert the bus (goes high)
        lda     #$ff            ; (2) clear our accumulated samples
$IF DEBUG
        ldx     #64             ; (2) 500us=64 loops x 19 clocks * 2.45MHz
$ELSEIF
        ldx     #211            ; (2) 500us=211 loops x 19 clocks * 8.00MHz
$ENDIF
irlp2:  and     portb           ; (3) merge new sample
        nsa                     ; (3) waste some time
        nsa                     ; (3) waste some more time
        nsa                     ; (3) waste some time
        nsa                     ; (3) waste some more time
        decx                    ; (1) decrement our counter
        bne     irlp2           ; (3) wait

        tax                     ; save A
        pula                    ; get CCR
        tap                     ; put in CCR
        txa                     ; get original A back
        and     #$01            ; set z bit if sampled bit (0) is clear
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      i_bit
; Called By:    isrch, i_writ, i_read, i_bit
; Calls:        None
; Gazintas:     A reg, bit 0: bit to send
; Gazoutas:     A reg, bit 0: bit received (rest shifted right)
; Function:     Here is where I transmit or receive a bit.  The same routine can
;               be used for both.  The low-going edge is the start.  A short
;               pulse is a 1, while a long pulse is a 0.  If you want to read,
;               send a short pulse and the slave will leave it alone for a 1 or
;               lengthen it for a 0.  Hence, this routine does both reading and
;               writing.  To read, send a 1, and if the value that comes back is
;               a 0, then that is the read value.
;               1 pulse is 1us wide
;               0 pulse is 60us wide
;               Data is sampled at 15us
;               The bit ends at 65us
;               The bit to send/receive is in the LSB of the A register
;
;               We used to disable interrupts while running this code, but now
;               that the code runs inside the interrupt routines, I've
;               commented this out.
; -----------------------------------------------------------------------------
i_bit:  tax                     ; Save A
        tpa                     ; get CCR
        psha                    ; save CCR
        txa                     ; get A back
        sei                     ; interrupts disabled

        bset    0,ddrb          ; (4) drop the data line
        bit     #$01            ; (2) test bit 0 of A
        beq     i_b0            ; (3) if 0, don't raise just yet
$IF DEBUG
        nsa                     ; (3) waste a bit of time in full-speed mode
        nsa                     ; (3) waste a bit of time in full-speed mode
$ENDIF
        bclr    0,ddrb          ; (4) raise the data line for a 1

i_b0:
$IF DEBUG
        ldx     #9              ; (2) delay count for 15us
$ELSEIF
        ldx     #30             ; (2) delay count for 15us
$ENDIF
idlp1:  decx                    ; (1) decrement our counter
        bne     idlp1           ; (3) wait

        brset   0,portb,i_na    ; (5) move portb0 to C (no branching)
i_na:   rora                    ; (3) put the bit from portb into A reg, bit 7

i_brx:
$IF DEBUG
        ldx     #28             ; (2) delay count for 45us
$ELSEIF
        ldx     #90             ; (2) delay count for 45us
$ENDIF
idlp2:  decx                    ; (1) decrement our counter
        bne     idlp2           ; (3) wait

        bclr    0,ddrb          ; (5) at 60us, clear no matter what

        jsr     mxiqen          ; (many) conditionally re-enable interrupts

;       Final delay is a bit more to guarantee some inter-bit high time.
$IF DEBUG
        ldx     #6              ; (2) delay count for 10us
$ELSEIF
        ldx     #20             ; (2) delay count for 10us
$ENDIF
idlp3:  decx                    ; (1) decrement our counter
        bne     idlp3           ; (3) wait

        tax                     ; save A
        pula                    ; get CCR
        tap                     ; put in CCR
        txa                     ; get original A back
        rts                     ; All done.

        page
; -----------------------------------------------------------------------------
; Routine:      i_writ, i_read
; Called By:    isrch,sndadr,iqtmp1,iqtmp2,iqwspd,iswitch,
; Calls:        i_bit (send/get a bit)
;               docrc (do 16-bt CRC)
; Gazintas:     A: byte to write (i_writ)
; Gazoutas:     A: byte got (i_read)
; Function:     These are the 1-wire read and write routines.  They are
;               essentially the same because of the open-collector nature of
;               the ibutton bus.  To read data all you do is "write" all 1's
;               and the reading device will convert bits on readback to 0's
;               where necessary.  The only reason I have seperate routines is
;               to avoid the initial #$ff in the calling code and that I do
;               the CRC processing in different places depending on the read
;               and write.   I am pretty sure I could consolidate these later
;               if need be.
; -----------------------------------------------------------------------------
i_writ: jsr     docrc           ; go compute the CRC before sending
        bsr     i_bit           ; send/Rx bit 0
        bsr     i_bit           ; send/Rx bit 1
        bsr     i_bit           ; send/Rx bit 2
        bsr     i_bit           ; send/Rx bit 3
        bsr     i_bit           ; send/Rx bit 4
        bsr     i_bit           ; send/Rx bit 5
        bsr     i_bit           ; send/Rx bit 6
        bsr     i_bit           ; send/Rx bit 7
        rts                     ; all done

i_read: lda     #$ff            ; for a read, want to "write" 1's
        bsr     i_bit           ; send/Rx bit 0
        bsr     i_bit           ; send/Rx bit 1
        bsr     i_bit           ; send/Rx bit 2
        bsr     i_bit           ; send/Rx bit 3
        bsr     i_bit           ; send/Rx bit 4
        bsr     i_bit           ; send/Rx bit 5
        bsr     i_bit           ; send/Rx bit 6
        bsr     i_bit           ; send/Rx bit 7
        jsr     docrc           ; with data Rx'd go CRC
        rts                     ; all done

; -----------------------------------------------------------------------------
; Routine:      i_rdb
; Called By:    isrch
; Calls:        i_bit
; Gazintas:     None
; Gazoutas:     Read bit A reg lsb
; Function:     Need to read a simple bit a lot, so rather than doing it the
;               hard way, I created a routine to initialize A to 1's (so I can
;               read), and go get the bit, but put it back in the A register
;               LSB where I need it.
; -----------------------------------------------------------------------------
i_rdb:  lda     #$ff            ; all 1's
        bsr     i_bit           ; go get a bit into A(0)
        rola                    ; result in C
        rts                     ; all done.

        page
; -----------------------------------------------------------------------------
; Routine:      i2srch
; Called By:    calwsp
;               calwnd
;               calhum
;               calbar
; Calls:        isrch
; Gazintas:     A : family code
; Gazoutas:     A : Number of devices at this family code (0,1,2 for now)
;               mndray : ID #1 if it exists
;               msg+20:msg+27 : ID #2 if it exists.
; Function:     This routine is used when we suspect there may be more than
;               one device with the same family code.  This will find up to
;               two devices (more is NASTY!!).  If only one is found, it is
;               stored at (mndray).  If there is a second, it is stored
;               at (msg+20).  If at least one sensor is found, (msg) is
;               altered.
; -----------------------------------------------------------------------------
i2srch: psha                    ; save family ID
        clr     flg12           ; start off one way...
        jsr     isrch           ; find an ID, put into mndray (A has family code)
        tst     mndray          ; make sure we got something
        bne     i2snxt          ; if so, on to see if there is a second

        pula                    ; pop ID off stack
        bra     i2s0            ; and exit with no IDs flag

i2snxt: ldhx    mndray          ; get ID bytes 1-2
        sthx    msg+20          ; save 'em
        ldhx    mndray+2        ; get ID bytes 3-4
        sthx    msg+22          ; save 'em
        ldhx    mndray+4        ; get ID bytes 5-6
        sthx    msg+24          ; save 'em
        ldhx    mndray+6        ; get ID bytes 7-8
        sthx    msg+26          ; save 'em

        inc     flg12           ; see if another counter ID
        pula                    ; get family ID from stack
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; make sure we got something
        beq     i2s0            ; if not, weird error, abort with 0 IDs

        ldhx    mndray          ; get 2nd reading bytes 1-2
        cphx    msg+20          ; see if same as 1st reading
        bne     i2s2            ; if different, two counters!
        ldhx    mndray+2        ; get 2nd reading bytes 3-4
        cphx    msg+22          ; see if same as 1st reading
        bne     i2s2            ; if different, two counters!
        ldhx    mndray+4        ; get 2nd reading bytes 5-6
        cphx    msg+24          ; see if same as 1st reading
        bne     i2s2            ; if different, two counters!
        ldhx    mndray+6        ; get 2nd reading bytes 7-8
        cphx    msg+26          ; see if same as 1st reading
        bne     i2s2            ; if different, two counters!
        bra     i2s1            ; if all same, go program it.

i2s2:   lda     #$02            ; flag 2 IDs
        bra     i2sex           ; all done
i2s1:   lda     #$01            ; flag 1 ID
        bra     i2sex           ; all done
i2s0:   clra                    ; flag NO ids
i2sex:  rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      i2pick
; Called By:    calwsp
;               calwnd
; Calls:        wrmess,wrstr,wrcmd,wrdat: write msg/string/command/data to LCD
;               wfkey : wait for keyboard press
; Gazintas:     (mndray) Chip ID #1
;               (msg+20) Chip ID #2
; Gazoutas:     (mndray) User-selected ID
;               cntr : trashed as a temporary variable
; Function:     This routine is used when the user needs to pick between two
;               IDs.  The two IDs referened above are displayed and the user
;               is asked to select which one is the correct one.  Whatever
;               the choice is is left in (mndray) for programming into the
;               flash.
; -----------------------------------------------------------------------------
i2pick: lda     #36             ; message #36 (Pick your sensor)
        jsr     wrmess          ; write it

        ldhx    #msg+20         ; point to first ID to convert to ASCII
        jsr     cnvid           ; go convert to ASCII in (msg+1)
        mov     #$94,msg        ; line 3
        ldhx    #msg            ; point to the whole message
        jsr     wrstr           ; go write it

        ldhx    #mndray         ; point to second ID to convert to ASCII
        jsr     cnvid           ; go convert to ASCII in (msg+1)
        mov     #$d4,msg        ; line 3
        ldhx    #msg            ; point to the whole message
        jsr     wrstr           ; go write it

        clr     cntr            ; our flag as to pick the ID

i2plp:  tst     cntr            ; test the A/B flag
        bne     i2bot           ; if not 0, B selection
itbtop: lda     #$a5            ; just after first ID text
        jsr     wrcmd           ; say there
        lda     #'*'            ; the marker
        jsr     wrdat           ; mark it
        lda     #$e5            ; just after second ID text
        jsr     wrcmd           ; say there
        lda     #' '            ; the NOT marker
        jsr     wrdat           ; mark it
        bra     i2bkey          ; on to user feedback

i2bot:  lda     #$a5            ; just after first ID text
        jsr     wrcmd           ; say there
        lda     #' '            ; the NOT marker
        jsr     wrdat           ; mark it
        lda     #$e5            ; just after second ID text
        jsr     wrcmd           ; say there
        lda     #'*'            ; the marker
        jsr     wrdat           ; mark it

i2bkey: jsr     wfnkey          ; wait for next keypress
        cmp     #ksel           ; see if select button (continue)
        beq     i2bpck          ; if so, go program selected ID
        cmp     #kup            ; see if up
        beq     i2bud           ; go process up/down (same)
        cmp     #kdown          ; see if up
        beq     i2bud           ; go process up/down (same)
        bra     i2bkey          ; if neither, ignore.

i2bud:  com     cntr            ; complement our A/B flag
        bra     i2plp           ; and go show that

i2bpck: tst     cntr            ; see which one user selected
        bne     i2bex           ; if B, requested ID is in (mndray), so done

        ldhx    msg+20          ; otherwise copy (msg+20) to (mndray)
        sthx    mndray          ; store bytes 0,1
        ldhx    msg+22          ; get bytes 2,3
        sthx    mndray+2        ; store 'em
        ldhx    msg+24          ; get bytes 2,3
        sthx    mndray+4        ; store 'em
        ldhx    msg+26          ; get bytes 2,3
        sthx    mndray+6        ; store 'em

i2bex:  rts                     ; done!
