        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         peet.asm
; Function:     The Peet output formatters.
; Routines:     ptmsg1: Sends first four fields of Peet string
;               ptmsg2: Sends second four fields of Peet string
;               ptmsg3: Sends third four fields of Peet string
;
; The Peet strings are weird enough to keep them seperate.  The minimum string
; to send out consists of 11 fields, five of which I don't support.
; Unsupported fields must be replaced with dashes, so half of my output string
; is dashes.  All output fields are in hexadecimal too instead of good old
; ASCII, so none of my APRS strings are really useful.  The intent behind
; seperating out this code is that it should have very little in common with
; the rest of my code and it has a different philosophy behind it.
;
; The minimum string is !!aaaabbbbccccddddeeeeffffgggghhhhiiiijjjjkkkkllll
; Sent by routine #     11111111111111111122222222222222223333333333333333
;
; !!   - is the header
; aaaa - is wind speed          (.1KPH)
; bbbb - is wind direction      (00-ff)
; cccc - is Temperature in      (.1 degrees F)
; dddd - is Long term rain      (---- for now)
; eeee - is Barometric pressure (in mb  xxxx.xin decimal, expressed in hex)
; ffff - is indoor temperature  (---- for now)
; gggg - is outside humidity    (---- for now)
; hhhh - is inside humidity     (---- for now)
; iiii - Date                   (day of year)
; jjjj - Time                   (minute of day)
; kkkk - Rain since Midnight    (100ths of an inch)
; llll - wind 1 minute avg      (.1KPH) (I'M DOING THE 5-MIN AVERAGE!!)
;
; Since the output is in ASCII hex and almost entirely non-standard units
; (.1KPH's, 0-255 for wind direction, date, time, etc), I am just going to
; keep all this code seperate and unique.
;
; The string is not sent out symetrically.  The initial string is shorter so
; that it can be sent completely before the second part of the string gets
; queued up on the second timer interrupt.  This means that the second string
; will actually take longer than 100ms to send, so UART interrupts will still
; be happening during the timer interrupt afterwards.  This should be OK.
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      ptmsg1
; Called By:    iqurt1
; Calls:        mul816 (8x16 unsigned multiply)
;               pthex  (converts 16-bit to ASCII hex at (hx))
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     errspd,winspd (Wind error flag and speed)
;               errdir,windir (Direction error flag and direction)
;               errtmp,ddtemp (Temperature error flag and temperature)
; Gazoutas:     msg (message to send to UART)
; Function:     This is the first of two routines to queue up data using the
;               Peet format.  This routine generates the first part of the
;               string: !!aaaabbbbccccdddd where
;               !!   - is the header
;               aaaa - is wind speed          (.1KPH)
;               bbbb - is wind direction      (00-ff)
;               cccc - is Temperature in      (.1 degrees F)
;               dddd - is Long term rain      (---- for now)
;
;               I describe how I convert each data item in its own header
;               below.
;
; -----------------------------------------------------------------------------
ptmsg1: lda     #'!'            ; the flag
        sta     msg             ; first one
        sta     msg+1           ; second one

        lda     #'-'            ; the default no-data flags
        clrh                    ; index MSB 0
        ldx     #19             ; want 21 dashes
ptm1lp: sta     msg+2,x         ; a dash
        decx                    ; decrement pointer
        bpl     ptm1lp          ; dash 'em all

; -----------------------------------------------------------------------------
; To convert wind speed from ticks to .1KPH's the magic number is 3.66618
; .1KPH's per tick.  I take the ticks, multiply, take the 18.6 format and
; normalize it to 16.8, convert to ASCII hex and plunk the data in the
; transmission string
; -----------------------------------------------------------------------------
        lda     errspd          ; get wind speed error counter
        and     #$c0            ; check both bits
        bne     ptmwd           ; if any errors, don't show

        lda     winspd          ; get current wind speed MSB
        sta     divdnd          ; conversion buffer
        lda     winspd+1        ; get cyrrebt wind speed LSB
        sta     divdnd+1        ; conversion buffer
        lda     #$db            ; 3.66618 in 2.6 format
        sta     divsor          ; store it
        jsr     mul816          ; convert 5-sec value to .1KPH

        lsl     quotnt+1        ; times 4 to normalize to 16.8 (2x)
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB
        lsl     quotnt+1        ; times 4 to normalize to 16.8 (4x)
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB

        lda     quotnt          ; put 16-bit value in quotnt
        sta     quotnt+1        ; move LSB
        lda     remndr          ; move MSB
        sta     quotnt          ; move MSB
        ldhx    #msg+2          ; point to the place to put the data
        jsr     pthex           ; plunk the data there

; -----------------------------------------------------------------------------
; For wind direction the format is a number from 0-255.  Since this we are using
; instantaneous wind direction values here this becomes trivial since our
; internal format is a number from 0-15.  We just (theoretically) multiply
; by 16 and plunk that result in the string.
; In practice, I simply fill the first, second, and last with ASCII 0, then
; take the wind direction, convert to ASCII and plunk in the third position.
; -----------------------------------------------------------------------------
ptmwd:  brset   7,errdir,ptmtmp ; if direction error, don't show data
        lda     #'0'            ; ASCII zero
        sta     msg+6           ; Wind direction MSB always 0
        sta     msg+7           ; rest of MSB
        sta     msg+9           ; LSN is 0 too!
        lda     windir          ; get wind direction # (0-15)
        and     #$0f            ; make sure its clean
        tax                     ; make an index
        clrh                    ; complete index
        lda     hextb,x         ; get ASCII value
        sta     msg+8           ; store it

; -----------------------------------------------------------------------------
; For temperatre we need to conver the degrees C temperature which is in 7.1
; format offset by 110 (55 degrees) and convert to .1 degrees F increments.
; The C*9/5+32 in real terms ends up being C*9+320 when you figger in all
; the scaling factors needed.
; -----------------------------------------------------------------------------
ptmtmp: brset   7,errdd,ptmltr  ; if error, don't show temperature
        lda     ddtemp          ; get current temperature
        sub     #110            ; convert to two's comp
        bpl     pttpos          ; if positive, no action required
        nega                    ; otherwise convert to positive
pttpos: ldx     #9              ; 9/5ths times 10 divided by two (see header)
        mul                     ; nearly .1 degrees F count
        stx     quotnt          ; save MSB
        sta     quotnt+1        ; save LSB

        lda     ddtemp          ; get raw temp back
        sub     #110            ; back to 1's comp to see if negative
        bpl     pttps2          ; if so, no action required

        neg     quotnt+1        ; otherwise negate LSB
        clra                    ; 0...
        sbc     quotnt          ; negate MSB
        sta     quotnt          ; save result

pttps2: lda     quotnt+1        ; get LSB
        add     #64             ; add 320 LSB (see header)
        sta     quotnt+1        ; save result
        lda     quotnt          ; get MSB
        adc     #1              ; add 320 MSB
        sta     quotnt          ; save result
        ldhx    #msg+10         ; point to the place to put the data
        jsr     pthex           ; plunk the data there

; -----------------------------------------------------------------------------
; For long term rain I get the difference between the reference value and
; the current value.
; -----------------------------------------------------------------------------
ptmltr: lda     rirgot          ; see if we have a rain guage
        bne     ptm1x           ; if not leave dashes
        lda     rancnt+1        ; get current rain counter LSB
        sub     firref+1        ; subtract reference LSB
        sta     quotnt+1        ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     firref          ; subtract reference MSB
        sta     quotnt          ; save result
        ldhx    #msg+14         ; the place for the result
        jsr     pthex           ; go put it there

; -----------------------------------------------------------------------------
; All done with part 1 of the message, go send it to the computer
; -----------------------------------------------------------------------------
ptm1x:  clr     msg+18          ; terminating character

        jsr     irqmsg          ; send the message
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      ptmsg2
; Called By:    iqurt2
; Calls:        pthex  (converts 16-bit to ASCII hex at (hx))
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     month, year, day, hours, mins (for time/date)
; Gazoutas:     msg (message to send to UART)
; Function:     This is the second of two routines to queue up data using the
;               Peet format.  This routine generates the second part of the
;               string eeeeffffgggghhhh where:
;               eeee - is Barometric pressure (eeee in Mb. if decimal would be xxxx.x)
;               ffff - is indoor temperature  (---- for now)
;               gggg - is outside humidity    (in .1% increments)
;               hhhh - is inside humidity     (---- for now)
; -----------------------------------------------------------------------------
ptmsg2: lda     #'-'            ; the default no-data flags
        clrh                    ; index MSB 0
        ldx     #15             ; want 16 dashes
ptm2lp: sta     msg,x           ; a dash
        decx                    ; decrement pointer
        bpl     ptm2lp          ; dash 'em all

; -----------------------------------------------------------------------------
; Leave dashes for barometric pressure, indoor temperature.
; fill in barometric pressure if it exists
; -----------------------------------------------------------------------------
        lda     fibar           ; get barometer ID first byte
        cmp     #$26            ; see if family code exists
        bne     ptmhum          ; if not $26, no sensor installed

        lda     errbar          ; get barometer error byte
        and     #$80            ; look at MSB only
        cmp     #$80            ; see if error set
        beq     ptmhum          ; if error, go to humidity

        lda     bavalm          ; get current barrometric pressure in Bavalm
        add     #$a3            ; add 931 (Barometric presssure = 931+Bavalm)
        sta     divdnd+1        ; save it
        lda     #$00            ; Load a with zero
        adc     #$03            ; finish add operation
        sta     divdnd          ; save it
        lda     #$0a            ; multiply by 10
        sta     divsor          ; save it
        jsr     mul816          ; mult by 10 (e.g. 980 becomes  9800
; MSB is now in Quotnt, LSB is in Quot+1     

; Now deal with tenth of Mb., located in Bavalm+1.  we are going to convert to a decimal
; number by dividing by 19h

        ldhx    #$0019          ; clear H, set divisor to 25 (19H)  divide ff/0a=19h 255/10=25.5
        lda     bavalm+1        ; get tenth data, set as dividend
        div                     ; do division
        add     quotnt+1        ; add it to LSB
        sta     quotnt+1        ; save it
        lda     quotnt          ; get MSB
        adc     #$0             ; account for carry if there is one
        sta     quotnt          ; save it

        ldhx    #msg            ; the place to put the answer
        jsr     pthex           ; plunk in the answer

; -----------------------------------------------------------------------------
; Fill in humidity if it exists.  I store the humidity in integer percent, so
; I need to multiply it by 10 before plunking it into the data string.
; -----------------------------------------------------------------------------
ptmhum: lda     fihum           ; get humidity ID first byte
        cmp     #$26            ; see if family code exists
        bne     pt2ex           ; if not $26, no sensor installed

        brset   7,errhum,pt2ex ; if humidity error, skip too

        lda     hmval           ; get current humidity
        and     #$7f            ; clear excess flag
        ldx     #10             ; wanna multiply by 10
        mul                     ; multiply it
        stx     quotnt          ; save MSB
        sta     quotnt+1        ; save LSB

        ldhx    #msg+8          ; the place to put the answer
        jsr     pthex           ; plunk in the answer

; -----------------------------------------------------------------------------
; Leave dashes for inside humidity.
; -----------------------------------------------------------------------------

pt2ex:  clr     msg+16          ; terminate the string
        jsr     irqmsg          ; send the message
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      ptmsg3
; Called By:    iqurt3
; Calls:        pthex  (converts 16-bit to ASCII hex at (hx))
;               irqmsg (enables IRQ-based UART message)
; Gazintas:     month, year, day, hours, mins (for time/date)
; Gazoutas:     msg (message to send to UART)
; Function:     This is the second of two routines to queue up data using the
;               Peet format.  This routine generates the second part of the
;               string iiiijjjjkkkkllll where:
;               iiii - Date                   (day of year)
;               jjjj - Time                   (minute of day)
;               kkkk - Rain since Midnight    (100ths of an inch)
;               llll - wind 1 minute avg      (.1KPH)
; -----------------------------------------------------------------------------
ptmsg3: lda     #'-'            ; the default no-data flags
        clrh                    ; index MSB 0
        ldx     #15             ; want 16 dashes
ptm3lp: sta     msg,x           ; a dash
        decx                    ; decrement pointer
        bpl     ptm3lp          ; dash 'em all

; -----------------------------------------------------------------------------
; For day of year, we first tally up all the days of the months before the
; current one and then add the current day-of-month.  I can't use the mthmax
; routine as it assumes the current month.
; -----------------------------------------------------------------------------
ptmdoy: clr     quotnt          ; (quotnt) is where we will tally
        clr     quotnt+1        ; clear both
        mov     #1,remndr       ; remndr is our month tallyer

ptmdlp: lda     remndr          ; get looped month
        cmp     month           ; see if equal to current month
        beq     ptmmx           ; if so, quit tallying
        tax                     ; make current month an index
        clrh                    ; index MSB 0
        lda     mthtbl,x        ; get days in month
        add     quotnt+1        ; add to tally
        sta     quotnt+1        ; save result
        inc     remndr          ; increment month count
        bcc     ptmdlp          ; no carry, on to next month
        inc     quotnt          ; otherwise increment MSB
        bra     ptmdlp          ; and on to next month

ptmmx:  lda     year            ; get year
        and     #$03            ; see if leap year
        bne     ptmnlp          ; if not x4, not a leap year
        lda     month           ; get current month
        cmp     #3              ; see if March or more
        bcs     ptmnlp          ; if not, don't add a day

        inc     quotnt+1        ; otherwise add a day
        bcc     ptmnlp          ; if no carry, done
        inc     quotnt          ; if so, increment MSB

ptmnlp: lda     day             ; get current day
        add     quotnt+1        ; add to tally
        sta     quotnt+1        ; store
        bcc     ptmdun          ; done if no carry
        inc     quotnt          ; otherwise increment MSB

ptmdun: ldhx    #msg+0          ; point to where to put the day
        jsr     pthex           ; and put it there

; -----------------------------------------------------------------------------
; Now for the minutes in the day.  This is simply hours times 60 plus minutes
; -----------------------------------------------------------------------------
        lda     hrs             ; get hours count (0-23)
        ldx     #60             ; 60 minutes per hour
        mul                     ; minutes so far based on hours
        stx     quotnt          ; save MSB
        add     mins            ; add minutes to tally
        sta     quotnt+1        ; save MSB
        bcc     ptmdx           ; if no carry, done
        inc     quotnt          ; otherwise increment MSB

ptmdx:  ldhx    #msg+4          ; point to where to put the day
        jsr     pthex           ; and put it there

; -----------------------------------------------------------------------------
; For rain since midnight, I get the randty value and plunk it in.
; -----------------------------------------------------------------------------
ptmstr: lda     rirgot          ; see if we have a rain guage
        bne     pt3spd          ; if not leave dashes
        brset   7,errran,pt3spd ; if sample error, leave dashes
        ldhx    rantdy          ; get todays rain accumulation
        sthx    quotnt          ; save it for conversion
        ldhx    #msg+8          ; the place for the result
        jsr     pthex           ; go put it there

; -----------------------------------------------------------------------------
; To convert wind speed from ticks to .1KPH's the magic number is 3.66618
; .1KPH's per tick.  I take the ticks, multiply, take the 18.6 format and
; normalize it to 16.8, convert to ASCII hex and plunk the data in the
; transmission string
; -----------------------------------------------------------------------------
pt3spd: lda     ptavws          ; get current wind speed MSB
        sta     divdnd          ; conversion buffer
        lda     ptavws+1        ; get cyrrebt wind speed LSB
        sta     divdnd+1        ; conversion buffer
        lda     #$db            ; 3.66618 in 2.6 format
        sta     divsor          ; store it
        jsr     mul816          ; convert 5-sec value to .1KPH

        lsl     quotnt+1        ; times 4 to normalize to 16.8 (2x)
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB
        lsl     quotnt+1        ; times 4 to normalize to 16.8 (4x)
        rol     quotnt          ; middle byte
        rol     remndr          ; MSB

        lda     quotnt          ; put 16-bit value in quotnt
        sta     quotnt+1        ; move LSB
        lda     remndr          ; move MSB
        sta     quotnt          ; move MSB
        ldhx    #msg+12         ; point to the place to put the data
        jsr     pthex           ; plunk the data there

; -----------------------------------------------------------------------------
; All done with part 3 of the message, go send it to the computer
; -----------------------------------------------------------------------------
ptm3x:  lda     #13             ; CR
        sta     msg+16          ; put it in the string
        lda     #10             ; LF
        sta     msg+17          ; put it in the string
        clr     msg+18          ; terminate the string

        jsr     irqmsg          ; send the message
        rts                     ; nuthin for now

        page
; -----------------------------------------------------------------------------
; Routine:      pthex
; Called By:    ptmsg1, ptmsg2
; Calls:        none
; Gazintas:     quotnt - 16-bit value to convert to ASCII
;               hx     - pointer to where to put answer
; Gazoutas:     (hx)   - 4-character ASCII hex version of quotnt
; Function:     This routine converts a 16-bit number to ASCII hex and puts
;               in (hx)-(hx+3).
;               I'm sure this could be more efficient with better stack
;               operations
; -----------------------------------------------------------------------------
pthex:  pshx                    ; save X
        pshh                    ; save H
        lda     quotnt          ; get binary MSB
        nsa                     ; get MSN in LSN
        and     #$0f            ; LSN only
        clrh                    ; index MSB 0
        tax                     ; make data index
        lda     hextb,x         ; get ASCII version
        pulh                    ; restore H
        pulx                    ; restore X
        sta     ,x              ; write byte

        pshx                    ; save X
        pshh                    ; save H
        lda     quotnt          ; get binary MSB
        and     #$0f            ; LSN only
        clrh                    ; index MSB 0
        tax                     ; make data index
        lda     hextb,x         ; get ASCII version
        pulh                    ; restore H
        pulx                    ; restore X
        sta     1,x             ; write byte

        pshx                    ; save X
        pshh                    ; save H
        lda     quotnt+1        ; get binary LSB
        nsa                     ; get MSN in LSN
        and     #$0f            ; LSN only
        clrh                    ; index MSB 0
        tax                     ; make data index
        lda     hextb,x         ; get ASCII version
        pulh                    ; restore H
        pulx                    ; restore X
        sta     2,x             ; write byte

        pshx                    ; save X
        pshh                    ; save H
        lda     quotnt+1        ; get binary LSB
        and     #$0f            ; LSN only
        clrh                    ; index MSB 0
        tax                     ; make data index
        lda     hextb,x         ; get ASCII version
        pulh                    ; restore H
        pulx                    ; restore X
        sta     3,x             ; write byte

        rts                     ; all done

