        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         tai603.asm
; Function:     Routines to communicate with the TAI603 weather head
; Routines:     t3rtp1: Read the temperature (stage 1)
;               t3rtp2: Read the temperature (stage 2)
;               t3rwsp: Read the wind speed
;               t3rwdr: Read the wind direction
;
; I decided to put all the routines for the TAI603 weather head into a seperate
; file to make them easier to find/debug.  Discovery of the weather head is
; done as part of the sensor discovery routine, so not here.  For each of the
; data gathering routines, I check for the TAI603s DS2750 family ID ($30) first,
; and if set, those routines branch to the routines here.  These routines will
; then get the data and return it in a form compatible with the way the data
; was read by the other routines.
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; Routines:     t3rtp1, t3rtp2
; Called By:    iqtmp1, iqtmp2
; Calls:        sndadr: Sends a 1-wire address
;               i_writ: writes a 1-wire byte
;               i_read: reads a 1-wire byte
; Gazintas:     None
; Gazoutas:     rdtemp: The temperature in C in 7.1 format
; Function:     This function reads the temperature from the DS2750 in the
;               TAI603 weather head.  Unfortunately, there is no error checking
;               over the 1-wire interface, so I simply read it twice and if
;               they don't compare I assume an error.  This will throw out
;               good data sometimes, but I don't have much else to go on.
;               The two reads occur at different times called at different times
;               by the sequencer.
;    =========> BTW, ERRORS DO OCCUR FREQUENTLY as the way the TAI603 weather head
;               works is that the data is in shared memory and the micro on
;               the weather head just grabs the 1-wire bus when it wants to
;               update memory, no negotiating!
; -----------------------------------------------------------------------------
t3rtp1: ldhx    #fitemp         ; address for DS2750
        jsr     sndadr          ; send it off
        lda     #$69            ; read memory command
        jsr     i_writ          ; send it
        lda     #$18            ; Address for temperature byte pair
        jsr     i_writ          ; send it

        jsr     i_read          ; read temperature MSB
        sta     rdtemp          ; save it for a bit
        rts                     ; done for now

t3rtp2: ldhx    #fitemp         ; address for DS2750 for the second read
        jsr     sndadr          ; send it off
        lda     #$69            ; read memory command
        jsr     i_writ          ; send it
        lda     #$18            ; Address for temperature byte pair
        jsr     i_writ          ; send it

        jsr     i_read          ; read temperature MSB
        cmp     rdtemp          ; compare to previously read value
        beq     t3rtok          ; if the same, process it, otherwise...

        inc     errtmp          ; increment the temperature error counter
        bset    7,errtmp        ; set MSB for error-now flag
        bra     t3rtex          ; and leave without doing stats

t3rtok: asla                    ; DS2750 is 8.0, DS1820 is 7.1, so left shift!
        add     #110            ; convert 2's comp to offset-110
        sta     rdtemp          ; save it
        inc     tsamp           ; increment temperature sample counter
t3rtex: rts                     ; all done!


; -----------------------------------------------------------------------------
; Routine :     t3rwsp
; Called By:
; Calls:        sndadr: Sends a 1-wire address
;               i_writ: writes a 1-wire byte
;               i_read: reads a 1-wire byte
; Gazintas:     None
; Gazoutas:     winspd: The wind speed in ticks per 5 seconds
; Function:     This function reads the wind speed data from the TAI603.  It
;               does a nice job of converting the speed in ticks per second to
;               meters per second, but I have to convert it back again as that
;               is what the rest of my code expects.
;
;               Lemme make sure things work the way I expect before documenting
;               too much here....
;
;               Magic Math based on AAG docs is to convert meters/sec back to
;               ticks per second is 0.912113552.  I then multiply that by two
;               since older sensors had two sensors per rev and then by 5 as
;               my other code got counts per 5 seconds.  So, 9.12113552 ticks
;               per meter per second or 0x92 inf 4.4 format.

;
;    =========> BTW, ERRORS DO OCCUR FREQUENTLY as the way the TAI603 weather head
;               works is that the data is in shared memory and the micro on
;               the weather head just grabs the 1-wire bus when it wants to
;               update memory, no negotiating!
; -----------------------------------------------------------------------------
t3rwsp: lda     errspd          ; get the error counter
        and     #$3f            ; clear out any older error bits
        sta     errspd          ; and save it

        ldhx    #fitemp         ; address for DS2750
        jsr     sndadr          ; send it off
        lda     #$69            ; read memory command
        jsr     i_writ          ; send it
        lda     #$88            ; Address for wx data from the micro
        jsr     i_writ          ; send it

        clr     lastws+3        ; our verfiy accumulator

        jsr     i_read          ; read the command byte
        sta     rdcnt1+0        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the wind speed
        sta     rdcnt1+1        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the wind direction
        sta     rdcnt1+2        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the light status
        sta     rdcnt1+3        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the light intensity
        sta     lastws+0        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the voltage
        sta     lastws+1        ; save it for a bit
        add     lastws+3        ; add to verify byte
        sta     lastws+3        ; and save it
        jsr     i_read          ; read the verify byte
        sta     lastws+2        ; save it for a bit
        cmp     lastws+3        ; see if verify bytes match
        bne     t3rwer          ; if not same, a comms error

        lda     rdcnt1          ; get the command byte
        cmp     #$a1            ; make sure it is the command byte we expect
        bne     t3rwer          ; if not, error

        lda     rdcnt1+1        ; get the wind speed in m/s
        ldx     #$92            ; multiplier to get ticks in 4.4 format
        mul                     ; get answer in 12.4 format

        lsrx                    ; downshift 12.4 format to 16.0
        rora                    ; Step 1 of 4
        lsrx                    ; downshift 12.4 format to 16.0
        rora                    ; Step 2 of 4
        lsrx                    ; downshift 12.4 format to 16.0
        rora                    ; Step 3 of 4
        lsrx                    ; downshift 12.4 format to 16.0
        rora                    ; Step 4 of 4
        bcc     t3rwnr          ; if last carry was clear, no rounding
        add     #1              ; need to add 1 instead of inc so C bit set!
        bcc     t3rwnr          ; if it didn't overflow, done
        incx                    ; otherwise increment X

t3rwnr: stx     winspd          ; save MSB
        sta     winspd+1        ; save LSB
        bra     t3rwex          ; and done

t3rwer: lda     errspd          ; get wind speed error counter
        inca                    ; increment wind speed error counter value
        and     #$3f            ; limit to 63
        ora     #$80            ; set error-now flags
        sta     errspd          ; save counter and flag
t3rwex: rts                     ; all done!

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

t3mclr: ldhx    #fitemp         ; address for DS2750
        jsr     sndadr          ; send it off
        lda     #$6c            ; write memory command
        jsr     i_writ          ; send it
        lda     #$88            ; Address for wx data from the micro
        jsr     i_writ          ; send it

        clra                    ; 0...
        jsr     i_writ          ; write it!

; -----------------------------------------------------------------------------
; Routines:     t3rwdr
; Called By:    iqdir3
; Calls:        None
; Gazintas:     None
; Gazoutas:     windir: The wind vane direction before north corrections.
; Function:     This routine returns the raw wind vane direction as reported
;               by the TAI603.  This turns out to be pretty easy as just a few
;               interrupts before.  The TAI603 returns both wind speed and
;               direction as part of the same packet of data and wind direction
;               is delivered (almost) in the format I need.  It returns a value
;               of 1-16 and I use a value of 0-15.  What value translates to
;               north is part of the calibration process and taken care of
;               elsewhere.
; -----------------------------------------------------------------------------
t3rwdr: tst     errspd          ; see if we got an error when I got wind speed
        bmi     t3rder          ; if so, no wind direction either!

        lda     rdcnt1+2        ; if no errors on previous read, get wind direction
        deca                    ; convert to 0-15
        and     #$0f            ; just in case...
        sta     windir          ; and save it

        bclr    7,errdir        ; make sure error flag is cleared
        bra     t3rdex          ; and exit

t3rder: inc     errdir          ; increment the wind direction error counter too
        bset    7,errdir        ; flag as bad data
        clr     windir          ; predictable value in event of error
t3rdex: rts                     ; all done!



