        page
******************************************************
** Copyright 2009 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         util.asm
; Function:     General purpose utilities
; Routines:     waits:  Waits 100ms times number in A reg
;               waitms: Waits 1ms times number in A reg
;               wfnkey: Waits for no key to be pressed.
;               divu:   Does 16-bit/8-bit division, 16r8 result
;               div16:  Does 16-bit/16-bit division, 16r0 result
;               mul816: 8x16 bit multiply (24-bit result)
;               bin2as: Convert binary byte to 3-digit ASCII
;               bin2am: Convert binary byte to 3-digit ASCII (mainline)
;               bin2ax: Convert binary word to 3-digit ASCII hex (mainline)
;               cnvid:  Convert 1-Wire ID to ASCII at (msg+1)
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      waits
; Called By:    losta places
; Calls:        waitms
; Gazintas:     A: number of 1/10ths of a second to wait
; Gazoutas:     None
; Function:     A time waster that is roughly calibrated in 1/10ths of a
;               second.
; -----------------------------------------------------------------------------
waits:  sta     dsecs           ; save the .1 sec counter
wsloop: lda     #100            ; .1 seconds = 100ms
        bsr     waitms          ; go wait that long
        dec     dsecs           ; decrement .1 seconds counter
        bne     wsloop          ; loop unitl zero
        rts

; -----------------------------------------------------------------------------
; Routine:      waitms
; Called By:    lotsa places
; Calls:        None
; Gazintas:     A: number of milliseconds to waste
; Gazoutas:     None
; Function:     A time waster that is roughly calibrated in milliseconds.
;               For the 6808 in debug mode, the bus speed is 2.4576MHz
;               For the 6808 in regular mode, the bus speed is 8.0MHz
; -----------------------------------------------------------------------------
waitms:
wmsolp: clrx                    ; (1) h:x to 0
        clrh                    ; (1) 2nd part
wmsilp: aix     #1              ; (2) increment
$IF DEBUG
        cphx    #306            ; (3) 1ms @8 2.45MHz clocks is 306 loops
$ELSEIF
        cphx    #1000           ; (3) 1ms @8 8.00MHz clocks is 1000 loops
$ENDIF
        bne     wmsilp          ; (3) if not, loop again
        deca                    ; (1) decrement millisecond counter
        bne     wmsolp          ; (3) loop for specified number of milliseconds
        clrh                    ; (1) leave H zero
        rts                     ; (4) All done

        page
; -----------------------------------------------------------------------------
; Routine:      wfnkey
; Called By:    lotsa places
; Calls:        None
; Gazintas:     None
; Gazoutas:     None
; Function:     This routine waits for no key to be pressed, then waits for
;               one to be pressed.  Pretty cheasy, but is needed in enough
;               places that a routine made sense.
; -----------------------------------------------------------------------------
wfnkey: lda     curkey          ; see if key
        bne     wfnkey          ; wait until no key pressed
wwwkey: lda     curkey          ; look again
        beq     wwwkey          ; wait until a key pressed
        rts

; -----------------------------------------------------------------------------
; Routine:      wnokey
; Called By:    lotsa places
; Calls:        None
; Gazintas:     None
; Gazoutas:     None
; Function:     This routine waits for no key to be pressed, then returns.
; -----------------------------------------------------------------------------
wnokey: lda     curkey          ; see if key
        bne     wnokey          ; wait until no key pressed
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      divu
; Called By:    lotsa places
; Calls:        None
; Gazintas:     divdnd (the 16-bit Dividend)
;               divsor (the 8-bit Divisor)
; Gazoutas:     quotnt (the 16-bit Quotient)
;               remndr (the 8-bit Remainder)
; Function:     This is a quick routine to do unsigned integer division.
;               This probably could be made more efficient, I'll do that later.
;               This was written before I knew about the 6808, but I think
;               this works more to what I need than the 6808 divide opcode.
; -----------------------------------------------------------------------------
divu:   lda     #16             ; Division is a 16-step process
        sta     temp            ; our step counter
        clra                    ; 8 bits of dividend clear

divlp:  lsl     divdnd+1        ; shift over dividend LSB
        rol     divdnd          ; rotate through MSB
        rola                    ; shift (current) MSB of dividend for compare
        cmp     divsor          ; compare to divisor
        blo     divles          ; if less, no action required (C set)
        sub     divsor          ; otherwise subtract divisor from dividend (C clear)
divles: rol     quotnt+1        ; C is inverted quotient bit, put in quotient
        rol     quotnt          ; rotate through entire quotient
        dec     temp            ; decrement shift/subtract counter
        bne     divlp           ; do all 16 bits

;       The carry bit after the compare is inverted from what I need, so I
;       invert the quotient bits.  The remainder is whatever is left after
;       the last shift-and-subtract, so I just copy that to the remainder
;       register.

        com     quotnt          ; invert MSB
        com     quotnt+1        ; invert LSB
        sta     remndr          ; store remainder
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      divf16
; Called By:    lotsa places
; Calls:        None
; Gazintas:     divdnd (the 16-bit Dividend)
;               divsor (the 16-bit Divisor MSB)
;               remndr (the 16-bit Divisor LSB)
; Gazoutas:     quotnt (the 16-bit Quotient)
; Function:     This is a quick routine to do unsigned integer division.
;               This code left-justifies both operators first to get a ratio.
;               Answer is in 1.15 format
; -----------------------------------------------------------------------------
divf16: tst     divsor          ; test divisor MSB
        bmi     divfdv          ; if MSB is 1, we're done shifting
        tst     divdnd          ; test dividend MSB
        bmi     divfdv          ; if MSB is 1, we're done shifting

        lsl     remndr          ; left justify divisor LSB
        rol     divsor          ; MSB too
        lsl     divdnd+1        ; left justify dividend LSB
        rol     divdnd          ; MSB too
        bra     divf16          ; see if we're done shifting

divfdv: lda     #16             ; Division is a 16-step process
        sta     temp            ; our step counter
        clr     quotnt          ; clear answer MSB
        clr     quotnt+1        ; clear answer LSB

divflp: lda     divdnd          ; get dividend MSB
        cmp     divsor          ; compare to divisor
        blo     divfna          ; if less, shift in a 0
        bhi     divfd1          ; if more, do subtract

        lda     divdnd+1        ; if MSBs the same, check LSBs
        cmp     remndr          ; do compare
        blo     divfna          ; if less, shift in a 0
                                ; more or same, do subtract

divfd1: lda     divdnd+1        ; get dividend LSB
        sub     remndr          ; subtract divisor LSB
        sta     divdnd+1        ; save result

        lda     divdnd          ; get dividend MSB
        sbc     divsor          ; subtract divisor MSB
        sta     divdnd          ; save result

        sec                     ; wanna shift in a 1
        bra     divfdl          ; done with this iteration

divfna: clc                     ; wanna shift in a 0

divfdl: rol     quotnt+1        ; shift answer into LSB
        rol     quotnt          ; shift answer into MSB

        lsr     divsor          ; divide divisor by two
        ror     remndr          ; LSB too

        dec     temp            ; decrement shift/subtract counter
        bne     divflp          ; do all 16 bits

        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      mul816
; Called By:    ptmsg1
; Calls:        none
; Gazintas:     divdnd - 16-bit number to get multiplied
;               divsor - 8-bit number to get muliplied
; Gazoutas:     remndr - MSB of 24-bit result
;               quotnt - middle and LSB of 24-bit result
; Function:     This routine does an 16-8 unsigned multiply.  Yes I know I'm
;               using names for the division routine, but it saves memory.
;               Maybe I'll pick better names later.
; -----------------------------------------------------------------------------
mul816: lda     divsor          ; get the multiplier
        ldx     divdnd+1        ; get multiplicand LSB
        mul                     ; multiply LSBs
        sta     quotnt+1        ; save LSB
        stx     quotnt          ; save MSB
        lda     divsor          ; get multiplier back
        ldx     divdnd          ; get multiplicand MSB
        mul                     ; multiply x256 value
        add     quotnt          ; add Middle-SBs
        sta     quotnt          ; save result
        txa                     ; get MSB in A
        adc     #0              ; add Carry if any
        sta     remndr          ; save 3-byte MSB
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      bin2as
; Called By:    INTERRUPT ROUTINES ONLY!!!
; Calls:        None
; Gazintas:     divdnd (16-bit binary number to convert)
;               zfill (0 to fill leading 0's, not 0 to lead with spaces.
; Gazoutas:     atemp (3-byte ASCII result)
; Function:     This routine converts a 16-bit binary number to ASCII.  It can
;               only handle numbers up to 999.  We convert by successively
;               subtracting 100's until the result is less than 100.  We then
;               subtracct 10's until the result is less than 10, then finally
;               do the units.  16-bit math is only required for the 100's as
;               after that all results are less than 255.
; -----------------------------------------------------------------------------
bin2as: lda     #'0'            ; default value
        sta     atemp+0         ; hundreds
        sta     atemp+1         ; tens
        sta     atemp+2         ; units

b2a100: lda     divdnd          ; get MSB
        bne     b2as1c          ; if non-zero, definitely more than 100
        lda     divdnd+1        ; if MSB=0, check LSB
        cmp     #100            ; see if >100
        blo     b2a10           ; if not, on to 10's

b2as1c: lda     divdnd+1        ; get LSB
        sub     #100            ; subtract 100 from number LSB
        sta     divdnd+1        ; save result
        bcc     b2as1d          ; if no carry, MSB unaffected
        dec     divdnd          ; otherwise decrement MSB
b2as1d: inc     atemp+0         ; increment ASCII 100's digit
        bra     b2a100          ; see if another 100

b2a10:  cmp     #10             ; see if >10
        blo     b2a1            ; if not, on to 1's
        sub     #10             ; subtract 10 from temperature
        inc     atemp+1         ; increment ASCII 10's digit
        bra     b2a10           ; see if anther 10

b2a1:   add     #'0'            ; convert units to ASCII
        sta     atemp+2         ; store it, done!

        tst     zfill           ; see if we are supposed to zero-fill
        bne     b2aex           ; if so, don't delete them

        lda     atemp           ; get temperature MSD
        cmp     #'0'            ; see if an ASCII 0
        bne     b2aex           ; if not, no 0's to kill
        lda     #' '            ; a space to replace the 0
        sta     atemp           ; make it so

        lda     atemp+1         ; get temperature 10's
        cmp     #'0'            ; see if an ASCII 0
        bne     b2aex           ; if not, no 0's to kill
        lda     #' '            ; a space to replace the 0
        sta     atemp+1         ; make it so

b2aex:  rts                     ; all done
        page
; -----------------------------------------------------------------------------
; Routine:      bin2am
; Called By:    MAINLINE ROUTINES ONLY
; Calls:        None
; Gazintas:     mbcnv (16-bit binary number to convert)
;               mzfill (0 to fill leading 0's, not 0 to lead with spaces.
; Gazoutas:     matemp (3-byte ASCII result)
; Function:     This is the mainline version of bin2as (no variable conflicts)
; -----------------------------------------------------------------------------
bin2am: lda     #'0'            ; default value
        sta     matemp+0        ; hundreds
        sta     matemp+1        ; tens
        sta     matemp+2        ; units

b2m100: lda     mbcnv           ; get MSB
        bne     b2ms1c          ; if non-zero, definitely more than 100
        lda     mbcnv+1         ; if MSB=0, check LSB
        cmp     #100            ; see if >100
        blo     b2m10           ; if not, on to 10's

b2ms1c: lda     mbcnv+1         ; get LSB
        sub     #100            ; subtract 100 from number LSB
        sta     mbcnv+1         ; save result
        bcc     b2ms1d          ; if no carry, MSB unaffected
        dec     mbcnv           ; otherwise decrement MSB
b2ms1d: inc     matemp+0        ; increment ASCII 100's digit
        bra     b2m100          ; see if another 100

b2m10:  cmp     #10             ; see if >10
        blo     b2m1            ; if not, on to 1's
        sub     #10             ; subtract 10 from temperature
        inc     matemp+1        ; increment ASCII 10's digit
        bra     b2m10           ; see if anther 10

b2m1:   add     #'0'            ; convert units to ASCII
        sta     matemp+2        ; store it, done!

        lda     mzfill          ; see if we are supposed to zero-fill
        bne     b2mex           ; if so, don't delete them

        lda     matemp          ; get temperature MSD
        cmp     #'0'            ; see if an ASCII 0
        bne     b2mex           ; if not, no 0's to kill
        lda     #' '            ; a space to replace the 0
        sta     matemp          ; make it so

        lda     matemp+1        ; get temperature 10's
        cmp     #'0'            ; see if an ASCII 0
        bne     b2mex           ; if not, no 0's to kill
        lda     #' '            ; a space to replace the 0
        sta     matemp+1        ; make it so

b2mex:  rts                     ; all done
        page
; -----------------------------------------------------------------------------
; Routine:      bin2ax
; Called By:    MAINLINE ROUTINES ONLY
; Calls:        None
; Gazintas:     mbcnv (16-bit binary number to convert)
; Gazoutas:     matemp (4-byte ASCII Hex result)
; Function:     This is the mainline version of bin2as (no variable conflicts)
; -----------------------------------------------------------------------------
bin2ax: lda     mbcnv           ; get MSB
        nsa                     ; get MSN in LSN
        and     #$0f            ; MSN only
        tax                     ; make an index
        clrh                    ; make sure index is right
        lda     hextb,x         ; convert to ASCII hex
        sta     matemp          ; save it

        lda     mbcnv           ; get MSB agin
        and     #$0f            ; LSN only
        tax                     ; make an index
        clrh                    ; make sure index is right
        lda     hextb,x         ; convert to ASCII hex
        sta     matemp+1        ; save it

        lda     mbcnv+1         ; get LSB
        nsa                     ; get MSN in LSN
        and     #$0f            ; MSN only
        tax                     ; make an index
        clrh                    ; make sure index is right
        lda     hextb,x         ; convert to ASCII hex
        sta     matemp+2        ; save it

        lda     mbcnv+1         ; get LSB agin
        and     #$0f            ; LSN only
        tax                     ; make an index
        clrh                    ; make sure index is right
        lda     hextb,x         ; convert to ASCII hex
        sta     matemp+3        ; save it

        rts                     ; all done



        page
; -----------------------------------------------------------------------------
; Routine:      cnvid
; Called By:    ssens
; Calls:        None
; Gazintas:     HX: pointer to ID to convert.
; Gazoutas:     (msg+1) 16-character ASCII version of ID
; Function:     This routine converts a 1-Wire ID to ASCII The ID is displayed
;               in the order the bytes are sent over the bus.  The contents are
;               stored starting at (msg+1) to allow a spot for a command byte
;               to be inserted before the string.
;               This code does not follow the standard easy way of using a
;               table for the character lookup as I have HX already busy.
; -----------------------------------------------------------------------------
cnvid:  lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+1          ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+3          ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+5          ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+7          ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+9          ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+11         ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        pshh                    ; save pointer to data to convert.
        pshx                    ; save LSB too
        ldhx    #msg+13         ; the place to put it
        bsr     cvid2d          ; go convert two digits

        pulx                    ; get ID pointer back
        pulh                    ; LSB too
        lda     ,x              ; get a byte to convert
        aix     #1              ; point to next
        ldhx    #msg+15         ; the place to put it
        bsr     cvid2d          ; go convert two digits
        clr     msg+17          ; terminate the string.

        rts                     ; done


cvid2d: psha                    ; save byte to convert
        nsa                     ; put MSN in LSN
        and     #$0f            ; 0-f only
        add     #'0'            ; 0-9 conversion
        cmp     #'9'            ; see if 0-9
        bls     cvid2x          ; if so, done
        add     #7              ; convert to a-f
cvid2x: sta     ,x              ; save ASCII digit
        pula                    ; get back full byte
        and     #$0f            ; 0-f only for LSN
        add     #'0'            ; 0-9 conversion
        cmp     #'9'            ; see if 0-9
        bls     cvid3x          ; if so, done
        add     #7              ; convert to a-f
cvid3x: sta     1,x             ; save ASCII digit
        rts                     ; done

        page

